@@ -1,15 +1,28 @@
 /* zip.c -- compress files to the gzip or pkzip format
- * Copyright (C) 1992-1993 Jean-loup Gailly
- * This is free software; you can redistribute it and/or modify it under the
- * terms of the GNU General Public License, see the file COPYING.
- */
 
-#ifndef lint
-static char rcsid[] = "$Id: zip.c,v 0.17 1993/06/10 13:29:25 jloup Exp $";
+   Copyright (C) 1997, 1998, 1999, 2006, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1992-1993 Jean-loup Gailly
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+
+#ifdef RCSID
+static char rcsid[] = "$Id: zip.c,v 1.7 2007/03/20 05:09:51 eggert Exp $";
 #endif
 
+#include <config.h>
 #include <ctype.h>
-#include <sys/types.h>
 
 #include "tailor.h"
 #include "gzip.h"
@@ -18,12 +31,12 @@ static char rcsid[] = "$Id: zip.c,v 0.17 1993/06/10 13:29:25 jloup Exp $";
 #ifdef HAVE_UNISTD_H
 #  include <unistd.h>
 #endif
-#ifndef NO_FCNTL_H
+#ifdef HAVE_FCNTL_H
 #  include <fcntl.h>
 #endif
 
 local ulg crc;       /* crc on uncompressed file data */
-long header_bytes;   /* number of bytes in gzip header */
+off_t header_bytes;   /* number of bytes in gzip header */
 
 /* ===========================================================================
  * Deflate in to out.
@@ -36,6 +49,7 @@ int zip(in, out)
     uch  flags = 0;         /* general purpose bit flags */
     ush  attr = 0;          /* ascii/binary flag */
     ush  deflate_flags = 0; /* pkzip -es, -en or -ex equivalent */
+    ulg  stamp;
 
     ifd = in;
     ofd = out;
@@ -52,7 +66,10 @@ int zip(in, out)
 	flags |= ORIG_NAME;
     }
     put_byte(flags);         /* general flags */
-    put_long(time_stamp);
+    stamp = (0 <= time_stamp.tv_sec && time_stamp.tv_sec <= 0xffffffff
+	     ? (ulg) time_stamp.tv_sec
+	     : (ulg) 0);
+    put_long (stamp);
 
     /* Write deflated file to zip file */
     crc = updcrc(0, 0);
@@ -65,12 +82,12 @@ int zip(in, out)
     put_byte(OS_CODE);            /* OS identifier */
 
     if (save_orig_name) {
-	char *p = basename(ifname); /* Don't save the directory part. */
+	char *p = gzip_base_name (ifname); /* Don't save the directory part. */
 	do {
 	    put_char(*p);
 	} while (*p++);
     }
-    header_bytes = (long)outcnt;
+    header_bytes = (off_t)outcnt;
 
     (void)deflate();
 
@@ -78,16 +95,15 @@ int zip(in, out)
   /* Check input size (but not in VMS -- variable record lengths mess it up)
    * and not on MSDOS -- diet in TSR mode reports an incorrect file size)
    */
-    if (ifile_size != -1L && isize != (ulg)ifile_size) {
-	Trace((stderr, " actual=%ld, read=%ld ", ifile_size, isize));
+    if (ifile_size != -1L && bytes_in != ifile_size) {
 	fprintf(stderr, "%s: %s: file size changed while zipping\n",
-		progname, ifname);
+		program_name, ifname);
     }
 #endif
 
     /* Write the crc and uncompressed size */
     put_long(crc);
-    put_long(isize);
+    put_long((ulg)bytes_in);
     header_bytes += 2*sizeof(long);
 
     flush_outbuf();
@@ -108,10 +124,14 @@ int file_read(buf, size)
 
     Assert(insize == 0, "inbuf not empty");
 
-    len = read(ifd, buf, size);
-    if (len == (unsigned)(-1) || len == 0) return (int)len;
+    len = read_buffer (ifd, buf, size);
+    if (len == 0) return (int)len;
+    if (len == (unsigned)-1) {
+	read_error();
+	return EOF;
+    }
 
     crc = updcrc((uch*)buf, len);
-    isize += (ulg)len;
+    bytes_in += (off_t)len;
     return (int)len;
 }
